#!/bin/bash

# sh makeJacobDet.sh
# Developed By: Kenneth I. Vaden, Jr. and Mark A. Eckert
# Edited: Mark A. Eckert, 2/25/2019

# Produce ANTS Jacobian determinant images (log) at the grossest
# resolution using the Step 1 ANTS normalization warps 

# Please note that we have indicated below where you should provide your 
# project-specific information with # EDIT HERE.
#########################################################################################################


echo "********************************************************************************************** "
echo "  META-WARP ALGORITHM INITIATED   "
echo "********************************************************************************************** "

DIM=3     # Dimensionality is 2 (jpg) or 3 (nii) - 2 is useful for fast parameter checking

# Make a directory for the metawarps
MetaWarpdir='/yourPath/images/Step01_Xforms/';                     # EDIT HERE.

# T1 Regularization file - successive template
REG_file='/yourPath/symAverage_Final_Step11.nii.gz';               # EDIT HERE 

# Filenames are generated from these specified values
T1_dir='/yourPath/images/';                                        # EDIT HERE.

# subject list
SUBJ_list='/yourPath/IDs.txt';                                     # EDIT HERE. subject IDs in nii filename

# Apply coarse ANTS warp settings to images with finer detail following each step
for x in `cat $SUBJ_list`; do   # Increment the counter, for each image

      cd ${MetaWarpdir}; echo cd ${MetaWarpdir};

      # FILENAMES
      Iprf='';                                                # EDIT HERE. add prefix, if necessary (e.g., 'r' for rigidly aligned images)
      BASE=${Iprf}${x}'Warp.nii.gz';     echo $BASE;          # EDIT HERE. remaining nii filename here + Warp.nii.gz (e.g., ${Iprf}${x-subjID}_T1_Warp.nii.gz)
      INNPT=${Iprf}${x}'';               echo $INNPT;         # EDIT HERE. remaining nii filename here

      # The warp and anatomy output filenames
      METAW='Step02'${BASE};
      OUTPT='Meta_Anat_Step02'${BASE}'.nii.gz';

      echo " ";
      echo "************************************************************ "
      echo "  PROCESSING SUBJECT # $BASE  ";
      echo " ";

# DF1 is the DIFF WARP to distort thru the pipeline.
      DF1=${T1_dir}'Step01_Xforms/Step01_'${INNPT}'Warp.nii.gz'

      # Affine and Diffeomorphic XFORMS:
      AF1=${T1_dir}'Step01_Xforms/Step01_'${INNPT}'Affine.txt' 
      AF2=${T1_dir}'Step02_Xforms/Step02_'${INNPT}'Affine.txt'
      DF2=${T1_dir}'Step02_Xforms/Step02_'${INNPT}'Warp.nii.gz'
      AF3=${T1_dir}'Step03_Xforms/Step03_'${INNPT}'Affine.txt'
      DF3=${T1_dir}'Step03_Xforms/Step03_'${INNPT}'Warp.nii.gz'

      # Combining warp fields (and affine parameters)
      # The Step 3 parameters are needed to align the images to the final 
      # normalization space but are not used to create the Step 1 Jacobian image
      XFORMS_ALL=${DF3}' '${AF3}' '${DF2}' '${AF2};
      XFORMS=${DF1}' '${AF1};
      echo ComposeMultiTransform 3 ${METAW} -R ${REG_file} ${XFORMS}; echo " ";
           ComposeMultiTransform 3 ${METAW} -R ${REG_file} ${XFORMS}; echo " ";

      # run ANTSJacobian if using older ANTS version
      # CreateJacobianDeterminantImage ImageDim gWarp outfile uselog maskfn normbytotalbool
      echo CreateJacobianDeterminantImage 3 ${METAW} AJ_Step01_${INNPT}logjacobian.nii.gz 1 none 0
           CreateJacobianDeterminantImage 3 ${METAW} AJ_Step01_${INNPT}logjacobian.nii.gz 1 none 0

      # Warp Jacobians to template space
      AJ_NATIVE=${MetaWarpdir}'AJ_Step01_'${INNPT}'logjacobian.nii.gz';
      AJ_WARPED=${MetaWarpdir}'wAJ_Step01_'${INNPT}'logjacobian.nii.gz';
      XFORMS=${METAW};
      echo ls ${AJ_NATIVE}; ls ${AJ_NATIVE};
      echo ls ${XFORMS_ALL}; ls ${XFORMS_ALL};
   
      echo WarpImageMultiTransform 3 ${AJ_NATIVE} ${AJ_WARPED} -R ${REG_file} ${XFORMS_ALL}; echo " ";
           WarpImageMultiTransform 3 ${AJ_NATIVE} ${AJ_WARPED} -R ${REG_file} ${XFORMS_ALL}; echo " ";

      # Move output files to a different directory
      mkdir jacobians;
      echo mv *AJ* ${T1_dir}Step01_Xforms/jacobians/; echo " ";       
           mv *AJ* ${T1_dir}Step01_Xforms/jacobians/; echo " ";    
   
done; echo " ";
      
echo "********************************************************************************************* "
echo "  META-WARP ALGORITHM COMPLETED   "
echo "********************************************************************************************* "
