#!/bin/bash

# sh makeJacobDet_Step03.sh
# Developed By: Kenneth I. Vaden, Jr. and Mark A. Eckert
# Edited: Mark A. Eckert, 2/25/2019

# Produce ANTS Jacobian determinant images (log) at the finest 
# resolution using the Step 3 ANTS normalization warps 

# Please note that we have indicated below where you should provide your 
# project-specific information with # EDIT HERE.
#########################################################################################################


echo "********************************************************************************************** "
echo "  META-WARP ALGORITHM INITIATED   "
echo "********************************************************************************************** "

DIM=3     # Dimensionality is 2 (jpg) or 3 (nii) - 2 is useful for fast parameter checking

# Make a directory for the metawarps
MetaWarpdir='/yourPath/images/Step03_Xforms/';                     # EDIT HERE.

# T1 Regularization file - successive template
REG_file='/yourPath/symAverage_Final_Step11.nii.gz';               # EDIT HERE.

# Filenames are generated from these specified values. 
T1_dir='/yourPath/images/';                                        # EDIT HERE.

# subject list
SUBJ_list='/yourPath/IDs.txt';                                     # EDIT HERE. subject IDs in nii filename

# Apply coarse ANTS warp settings to images with finer detail following each step
for x in `cat $SUBJ_list`; do   # Increment the counter, for each image

      cd ${MetaWarpdir}; echo cd ${MetaWarpdir};

      # FILENAMES
      Iprf='';                                                # EDIT HERE. add prefix, if necessary (e.g., 'r' for rigidly aligned images)
      BASE=${Iprf}${x}'Warp.nii.gz';     echo $BASE;          # EDIT HERE. remaining nii filename here + Warp.nii.gz (e.g., ${Iprf}${x-subjID}_T1_Warp.nii.gz)
      INNPT=${Iprf}${x}'';               echo $INNPT;         # EDIT HERE. remaining nii filename here

      # The warp and anatomy output filenames
      METAW='Step02'${BASE};
      OUTPT='Meta_Anat_Step02'${BASE}'.nii.gz';

      echo " ";
      echo "************************************************************ "
      echo "  PROCESSING SUBJECT # $BASE  ";
      echo " ";

      # DF1 is the DIFF WARP to distort thru the pipeline.
      DF1=${T1_dir}'Step01_Xforms/Step01_'${INNPT}'Warp.nii.gz'

      # Affine and Diffeomorphic XFORMS:
      AF1=${T1_dir}'Step01_Xforms/Step01_'${INNPT}'Affine.txt' 
      AF2=${T1_dir}'Step02_Xforms/Step02_'${INNPT}'Affine.txt'
      DF2=${T1_dir}'Step02_Xforms/Step02_'${INNPT}'Warp.nii.gz'
      AF3=${T1_dir}'Step03_Xforms/Step03_'${INNPT}'Affine.txt'
      DF3=${T1_dir}'Step03_Xforms/Step03_'${INNPT}'Warp.nii.gz'

      # Combining warp fields (and affine parameters)
      XFORMS=${DF3}' '${AF3};
      echo ComposeMultiTransform 3 ${METAW} -R ${REG_file} ${XFORMS}; echo " ";
           ComposeMultiTransform 3 ${METAW} -R ${REG_file} ${XFORMS}; echo " ";

      # run ANTSJacobian if using older ANTS version
      # CreateJacobianDeterminantImage ImageDim gWarp outfile uselog maskfn normbytotalbool
      echo CreateJacobianDeterminantImage 3 ${METAW} wAJ_Step03_${INNPT}.nii.gz 1 none 0
           CreateJacobianDeterminantImage 3 ${METAW} wAJ_Step03_${INNPT}.nii.gz 1 none 0

      # Move output files to a different directory
      mkdir jacobians;
      echo mv *AJ* ${T1_dir}Step03_Xforms/jacobians/; echo " "; 
           mv *AJ* ${T1_dir}Step03_Xforms/jacobians/; echo " "; 

      # Warp Jacobians to template space - NOT needed for Step 3 warps b/c they are already in template space
      # (see metaWarp_Global.sh and metaWarp_Local.sh, which warp the Jacobians)

done; echo " ";
      
echo "********************************************************************************************* "
echo "  META-WARP ALGORITHM COMPLETED   "
echo "********************************************************************************************* "
